@@ -24,6 +24,7 @@
 
 #include <QApplication>
 #include <QClipboard>
+#include <QCommandLineParser>
 #include <QDir>
 #include <QDrag>
 #include <QKeySequence>
@@ -86,6 +87,7 @@ void SpectacleCore::onActivateRequested(QStringList arguments, const QString & /
     parser->parse(arguments);
 
     mStartMode = SpectacleCore::StartMode::Gui;
+    mExistingLoaded = false;
     mNotify = true;
     qint64 lDelayMsec = 0;
 
@@ -96,6 +98,18 @@ void SpectacleCore::onActivateRequested(QStringList arguments, const QString & /
         mStartMode = SpectacleCore::StartMode::DBus;
     }
 
+    mEditExisting = parser->isSet(QStringLiteral("edit-existing"));
+    if (mEditExisting) {
+        QString lExistingFileName = parser->value(QStringLiteral("edit-existing"));
+        if (!(lExistingFileName.isEmpty() || lExistingFileName.isNull())) {
+            if (QDir::isRelativePath(lExistingFileName)) {
+                lExistingFileName = QDir::current().absoluteFilePath(lExistingFileName);
+            }
+            setFilename(lExistingFileName);
+            mSaveToOutput = true;
+        }
+    }
+
     auto lOnClickAvailable = mPlatform->supportedShutterModes().testFlag(Platform::ShutterMode::OnClick);
     if ((!lOnClickAvailable) && (lDelayMsec < 0)) {
         lDelayMsec = 0;
@@ -121,10 +135,11 @@ void SpectacleCore::onActivateRequested(QStringList arguments, const QString & /
     } else if (parser->isSet(QStringLiteral("transientonly"))) {
         lCaptureMode = Spectacle::CaptureMode::WindowUnderCursor;
     } else if (mStartMode == SpectacleCore::StartMode::Gui
-               && (parser->isSet(QStringLiteral("launchonly")) || Settings::onLaunchAction() == Settings::EnumOnLaunchAction::DoNotTakeScreenshot)) {
+               && (parser->isSet(QStringLiteral("launchonly")) || Settings::onLaunchAction() == Settings::EnumOnLaunchAction::DoNotTakeScreenshot)
+               && !mEditExisting) {
         initGuiNoScreenshot();
         return;
-    } else if (Settings::onLaunchAction() == Settings::EnumOnLaunchAction::UseLastUsedCapturemode) {
+    } else if (Settings::onLaunchAction() == Settings::EnumOnLaunchAction::UseLastUsedCapturemode && !mEditExisting) {
         lCaptureMode = Settings::captureMode();
         if (Settings::onClickChecked()) {
             lDelayMsec = -1;
@@ -146,11 +161,19 @@ void SpectacleCore::onActivateRequested(QStringList arguments, const QString & /
     case StartMode::Background: {
         mCopyImageToClipboard = false;
         mCopyLocationToClipboard = false;
+        mSaveToOutput = true;
 
         if (parser->isSet(QStringLiteral("nonotify"))) {
             mNotify = false;
         }
 
+        if (parser->isSet(QStringLiteral("copy-image"))) {
+            mSaveToOutput = false;
+            mCopyImageToClipboard = true;
+        } else if (parser->isSet(QStringLiteral("copy-path"))) {
+            mCopyLocationToClipboard = true;
+        }
+
         if (parser->isSet(QStringLiteral("output"))) {
             mSaveToOutput = true;
             QString lFileName = parser->value(QStringLiteral("output"));
@@ -174,12 +197,6 @@ void SpectacleCore::onActivateRequested(QStringList arguments, const QString & /
             lDelayMsec = -1;
         }
 
-        if (parser->isSet(QStringLiteral("copy-image"))) {
-            mCopyImageToClipboard = true;
-        } else if (parser->isSet(QStringLiteral("copy-path"))) {
-            mCopyLocationToClipboard = true;
-        }
-
         if (!mIsGuiInited) {
             static_cast<QGuiApplication *>(qApp->instance())->setQuitOnLastWindowClosed(false);
         }
@@ -316,6 +333,12 @@ void SpectacleCore::screenshotsUpdated(const QVector<QImage> &imgs)
 
 void SpectacleCore::screenshotUpdated(const QPixmap &thePixmap)
 {
+    QPixmap existingPixmap;
+    const QPixmap &pixmapUsed = (mEditExisting && !mExistingLoaded) ? existingPixmap : thePixmap;
+    if (mEditExisting && !mExistingLoaded) {
+        existingPixmap.load(filename());
+    }
+
     auto lExportManager = ExportManager::instance();
 
     if (lExportManager->captureMode() == Spectacle::CaptureMode::RectangularRegion) {
@@ -325,7 +348,7 @@ void SpectacleCore::screenshotUpdated(const QPixmap &thePixmap)
         }
     }
 
-    lExportManager->setPixmap(thePixmap);
+    lExportManager->setPixmap(pixmapUsed);
     lExportManager->updatePixmapTimestamp();
 
     switch (mStartMode) {
@@ -357,12 +380,12 @@ void SpectacleCore::screenshotUpdated(const QPixmap &thePixmap)
         }
     } break;
     case StartMode::Gui:
-        if (thePixmap.isNull()) {
-            mMainWindow->setScreenshotAndShow(thePixmap);
+        if (pixmapUsed.isNull()) {
+            mMainWindow->setScreenshotAndShow(pixmapUsed, false);
             mMainWindow->setPlaceholderTextOnLaunch();
             return;
         }
-        mMainWindow->setScreenshotAndShow(thePixmap);
+        mMainWindow->setScreenshotAndShow(pixmapUsed, mEditExisting);
 
         bool autoSaveImage = Settings::autoSaveImage();
         mCopyImageToClipboard = Settings::clipboardGroup() == Settings::EnumClipboardGroup::PostScreenshotCopyImage;
@@ -378,6 +401,12 @@ void SpectacleCore::screenshotUpdated(const QPixmap &thePixmap)
             lExportManager->doCopyLocationToClipboard(false);
         }
     }
+
+    if (mEditExisting && !mExistingLoaded) {
+        Settings::setLastSaveLocation(mFileNameUrl);
+        mMainWindow->imageSaved(mFileNameUrl);
+        mExistingLoaded = true;
+    }
 }
 
 void SpectacleCore::screenshotCanceled()
@@ -385,7 +414,7 @@ void SpectacleCore::screenshotCanceled()
     mQuickEditor->hide();
     mQuickEditor.reset(nullptr);
     if (mStartMode == StartMode::Gui) {
-        mMainWindow->setScreenshotAndShow(QPixmap());
+        mMainWindow->setScreenshotAndShow(QPixmap(), false);
     } else {
         Q_EMIT allDone();
     }
@@ -409,7 +438,7 @@ void SpectacleCore::screenshotFailed()
         return;
     case StartMode::Gui:
         mMainWindow->screenshotFailed();
-        mMainWindow->setScreenshotAndShow(QPixmap());
+        mMainWindow->setScreenshotAndShow(QPixmap(), false);
     }
 }
 
@@ -439,36 +468,39 @@ void SpectacleCore::doNotify(const QUrl &theSavedAt)
         break;
     }
 
-    // a speaking message is prettier than a URL, special case for copy to clipboard and the default pictures location
+    // a speaking message is prettier than a URL, special case for copy image/location to clipboard and the default pictures location
     const QString &lSavePath = theSavedAt.adjusted(QUrl::RemoveFilename | QUrl::StripTrailingSlash).path();
 
-    static bool isBackgroundMode = (mStartMode == SpectacleCore::StartMode::Background);
-
-    if (mSaveToOutput || isBackgroundMode) {
-        lNotify->setText(i18n("A screenshot was saved as '%1' to '%2'.", theSavedAt.fileName(), lSavePath));
-        // set to false so it won't show the same message twice
-        isBackgroundMode = false;
-    } else if (mCopyImageToClipboard) {
+    if (mCopyImageToClipboard && theSavedAt.fileName().isEmpty()) {
         lNotify->setText(i18n("A screenshot was saved to your clipboard."));
+    } else if (mCopyLocationToClipboard && !theSavedAt.fileName().isEmpty()) {
+        lNotify->setText(i18n("A screenshot was saved as '%1' to '%2' and the file path of the screenshot has been saved to your clipboard.",
+                              theSavedAt.fileName(),
+                              lSavePath));
     } else if (lSavePath == QStandardPaths::writableLocation(QStandardPaths::PicturesLocation)) {
         lNotify->setText(i18nc("Placeholder is filename", "A screenshot was saved as '%1' to your Pictures folder.", theSavedAt.fileName()));
-    } else {
+    } else if (!theSavedAt.fileName().isEmpty()) {
         lNotify->setText(i18n("A screenshot was saved as '%1' to '%2'.", theSavedAt.fileName(), lSavePath));
     }
 
     if (!theSavedAt.isEmpty()) {
         lNotify->setUrls({theSavedAt});
         lNotify->setDefaultAction(i18nc("Open the screenshot we just saved", "Open"));
-        connect(lNotify, QOverload<uint>::of(&KNotification::activated), this, [this, theSavedAt](uint index) {
-            if (index == 0) {
-                auto job = new KIO::OpenUrlJob(theSavedAt);
-                job->start();
-                QTimer::singleShot(250, this, [this] {
-                    if (!mIsGuiInited || Settings::quitAfterSaveCopyExport()) {
-                        Q_EMIT allDone();
-                    }
-                });
-            }
+        connect(lNotify, &KNotification::defaultActivated, this, [this, theSavedAt]() {
+            auto job = new KIO::OpenUrlJob(theSavedAt);
+            job->start();
+            QTimer::singleShot(250, this, [this] {
+                if (!mIsGuiInited || Settings::quitAfterSaveCopyExport()) {
+                    Q_EMIT allDone();
+                }
+            });
+        });
+        lNotify->setActions({i18n("Annotate")});
+        connect(lNotify, &KNotification::action1Activated, this, [this, theSavedAt]() {
+            QProcess newInstance;
+            newInstance.setProgram(QStringLiteral("spectacle"));
+            newInstance.setArguments({QStringLiteral("--new-instance"), QStringLiteral("--edit-existing"), theSavedAt.toLocalFile()});
+            newInstance.startDetached();
         });
     }
 
@@ -506,18 +538,22 @@ void SpectacleCore::populateCommandLineParser(QCommandLineParser *lCmdLineParser
         {{QStringLiteral("d"), QStringLiteral("delay")},
          i18n("In background mode, delay before taking the shot (in milliseconds)"),
          QStringLiteral("delayMsec")},
-        {{QStringLiteral("c"), QStringLiteral("copy-image")}, i18n("In background mode, copy screenshot image to clipboard")},
+        {{QStringLiteral("c"), QStringLiteral("copy-image")}, i18n("In background mode, copy screenshot image to clipboard, unless -o is also used.")},
         {{QStringLiteral("C"), QStringLiteral("copy-path")}, i18n("In background mode, copy screenshot file path to clipboard")},
         {{QStringLiteral("w"), QStringLiteral("onclick")}, i18n("Wait for a click before taking screenshot. Invalidates delay")},
         {{QStringLiteral("i"), QStringLiteral("new-instance")}, i18n("Starts a new GUI instance of spectacle without registering to DBus")},
         {{QStringLiteral("p"), QStringLiteral("pointer")}, i18n("In background mode, include pointer in the screenshot")},
         {{QStringLiteral("e"), QStringLiteral("no-decoration")}, i18n("In background mode, exclude decorations in the screenshot")},
+        {{QStringLiteral("E"), QStringLiteral("edit-existing")}, i18n("Open and edit existing screenshot file"), QStringLiteral("existingFileName")},
     });
 }
 
 void SpectacleCore::doStartDragAndDrop()
 {
     auto lExportManager = ExportManager::instance();
+    if (lExportManager->pixmap().isNull()) {
+        return;
+    }
     QUrl lTempFile = lExportManager->tempSave();
     if (!lTempFile.isValid()) {
         return;
